package com.portlandwebworks.commons.dao;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class FieldProjection
{
	private String entityPath;
	
	private String beanPath;

	public FieldProjection()
	{
	}

	public FieldProjection(String entityPath, String beanPath)
	{
		this.entityPath = entityPath;
		this.beanPath = beanPath;
	}

	@Override
	public String toString()
	{
		return new StringBuilder("FieldProjection(beanPath=").append(beanPath).append(", entityPath=").append(entityPath).append(")").toString();
	}

	/**
	 * Builds a list of field projections with the same bean path as the entity path.  Useful for populating a Map
	 * of projected query results.
	 *
	 * @param fieldPaths
	 * @return
	 */
	public static FieldProjection[] all(String... fieldPaths)
	{
		FieldProjection[] projections = new FieldProjection[fieldPaths.length];
		
		for (int i = 0; i < fieldPaths.length; i++) {
			projections[i] = new FieldProjection(fieldPaths[i], fieldPaths[i]);
		}
		
		return projections;
	}

	public String getEntityPath()
	{
		return entityPath;
	}

	public void setEntityPath(String entityPath)
	{
		this.entityPath = entityPath;
	}

	public String getBeanPath()
	{
		return beanPath;
	}

	public void setBeanPath(String beanPath)
	{
		this.beanPath = beanPath;
	}
}
