/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.dao;

/**
 * This software is the property of Portland Webworks.
 *
 * For use with a @SearchCriterion annotation.
 *
 * @see com.portlandwebworks.commons.dao.SearchCriterion
 *
 * @Author rnorthup
 */
public enum Operator
{
	/**
	 * A single operand is equal to a single criterion value.
	 */
	EqualTo,

	/**
	 * A single operand is not equal to a single criterion value.
	 */
	NotEqualTo,

	/**
	 * A text-based operand contains the substring represented by the criterion value.
	 *
	 * @deprecated  Use Contains instead.  I decided Matches was misleading, sounds like equals
	 */
	@Deprecated
	Matches,

	MatchesAnywhere,

	/**
	 * A text-based operand matches the beginning of the string represented by the criterion value
	 */
	MatchesStart,

	/**
	 * 
	 */
	MatchesEnd,

	/**
	 * A scalar operand is greater than the criterion value.
	 */
	GreaterThan,

	/**
	 * A scalar operand is less than the criterion value.
	 */
	LessThan,

	/**
	 * A scalar operand is greater than or equal to the criterion value.
	 */
	GreaterThanOrEqual,

	/**
	 * A scalar operand is less than or equal to the criterion value.
	 */
	LessThanOrEqual,

	/**
	 * An operand is null.
	 */
	IsNull,

	/**
	 * An operand is not null.
	 */
	IsNotNull,

	/**
	 * A single-value operand is one of the criterion values.
	 */
	IsIn,

	/**
	 * For aggregate criterion values, the count of aggregate items is equal to.
	 */
	CountEqualTo,

	/**
	 * For aggregate criterion values, the count of aggregate items is not equal to.
	 */
	CountNotEqualTo,

	/**
	 * For aggregate criterion values, the count of aggregate items is greater than.
	 */
	CountGreaterThan,

	/**
	 * For aggregate criterion values, the count of aggregate items is greater than or equal to.
	 */
	CountGreaterThanOrEqual,

	/**
	 * For aggregate criterion values, the count of aggregate items is less than.
	 */
	CountLessThan,

	/**
	 * For aggregate criterion values, the count of aggregate items is less than or equal to.
	 */
	CountLessThanOrEqual,

	/**
	 * uses % or * characters
	 */
	IsLike

}