package com.portlandwebworks.commons.dao;

import org.apache.commons.collections.Factory;
import org.apache.commons.collections.list.LazyList;

import java.util.ArrayList;
import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * A simplification of the default IPagedSearchOption which handles sorting on a single property.  It will provide the
 * caller with a list of sort specifications containing either one or zero elements.  It also provides bean properties
 * for setting a singular sort specification's sortProperty and sortDirection fields.
 *
 * @author Ronnie Northup
 */
public class SimplePagedSearchOptions implements IPagedSearchOptions
{
	private int start;

	private int limit;

	private List<SortSpecification> sort;

	public SimplePagedSearchOptions()
	{
		start = 0;
		limit = Integer.MAX_VALUE;
	}

	public SimplePagedSearchOptions(int start, int limit)
	{
		this.start = start;
		this.limit = limit;
	}

	public SimplePagedSearchOptions(int start, int limit, String sortProperty, SortDirection sortDirection)
	{
		this(start, limit);

		sort = new ArrayList<SortSpecification>();
		sort.add(new SortSpecification(sortProperty, sortDirection));
	}

	public List<SortSpecification> getSort()
	{
		if (sort == null) {
			sort = LazyList.decorate(new ArrayList<SortSpecification>(), new Factory()
			{
				public Object create()
				{
					return new SortSpecification();
				}
			});
		}

		return sort;
	}

	public int getStart()
	{
		return start;
	}

	public void setStart(int start)
	{
		this.start = start;
	}

	public int getLimit()
	{
		return limit;
	}

	public void setLimit(int limit)
	{
		this.limit = limit;
	}

	public String getSortProperty()
	{
		List<SortSpecification> list = this.getSort();
		if (list.size() == 0) {
			return null;
		}
		return list.get(0).getProperty();
	}

	public void setSortProperty(String sortProperty)
	{
		this.getSort().get(0).setProperty(sortProperty);
	}

	public SortDirection getSortDirection()
	{
		List<SortSpecification> list = this.getSort();
		if (list.size() == 0) {
			return null;
		}
		return list.get(0).getDirection();
	}

	public void setSortDirection(SortDirection sortDirection)
	{
		this.getSort().get(0).setDirection(sortDirection);
	}
}
