package com.portlandwebworks.commons.data;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This software is the property of Portland Webworks.
 *
 * @author rnorthup
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.FIELD})
public @interface FieldMapping
{
	/**
	 * The path to the entity field to which this field's value is to be applied.
	 *
	 * @return
	 */
	public String value() default "";

	/**
	 * The path to an alternate entity field whos value should be used if the field's value is null.
	 * @return
	 */
	public String ifNullValue() default "";

	/**
	 * This doesn't currently do anything
	 *
	 * @return
	 */
	public boolean unique() default false;

	/**
	 * Indicates this field is the unique identifier for the entity to which the field's owner maps to.
	 * 
	 * @return
	 */
	public boolean identifier() default false;

	/**
	 * The name of a spring bean that will convert this field's value to its final format.
	 *
	 * @return
	 */
	public String transformer() default "";

	/**
	 * When updating an entity from a mapped object, do not apply this field's value to the entity's mapped field.  This
	 * is useful for including fields in an input object that are informational only and should not be used to populate
	 * a domain object during entity upates.
	 * 
	 * @return
	 */
	public boolean readOnly() default false;

	/**
	 * This field will be ignored on all updates except those in which a matching group value is passed to the entity
	 * builder during an update.  This allows wizard style "stepped" data entry, as well as only updating certain fields
	 * when a particular user role is performing the update.
	 *
	 * @return
	 */
	public String[] groups() default "";

	/**
	 * When this field has a null value, do not apply it to the entity's mapped field.
	 *
	 * @return
	 */
	public boolean ignoreNull() default false;

	/**
	 * Allows mapping a field to a generic type such as a lookup value
	 */
	public String mappedClass() default "";

	/**
	 * When set to true on a FieldMapping for a nested object, the entity builder will create a new nested entity if no
	 * identifier is provided for an existing entity.
	 *
	 * @return
	 */
	public boolean autoCreate() default false;
}
