/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.domain;

import com.portlandwebworks.commons.dao.DefaultOrderBy;
import org.hibernate.annotations.Type;

import javax.persistence.*;
import java.util.Date;

/**
 * This software is the property of Portland Webworks.
 *
 * This subclass of EntityBase provides additional common fields to provide auditing capabilities.
 *
 * The default sorting behavior of this type is to order first by last modified date descending, then by created date
 * descending.  You can override default sorting by specifying @DefaultOrderBy annotations on your subclass fields
 * with priority values less than 100.
 *
 * @Author rnorthup
 */
@MappedSuperclass
public abstract class AuditedEntity<T extends User> extends EntityBase
{
	@Column(name = "created_date")
	@Type(type = "com.portlandwebworks.commons.hibernate.DateTimeType")
	@DefaultOrderBy(priority = 110, ascending = false)
	private Date createdDate;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "creating_user_id")
	private T creatingUser;

	@Column(name = "last_modified_date")
	@Type(type = "com.portlandwebworks.commons.hibernate.DateTimeType")
	@DefaultOrderBy(priority = 100, ascending = false)
	private Date lastModifiedDate;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "last_modifying_user_id")
	private T lastModifyingUser;

	

	public Date getCreatedDate()
	{
		return createdDate;
	}

	public void setCreatedDate(Date createdDate)
	{
		this.createdDate = createdDate;
	}

	public T getCreatingUser()
	{
		return creatingUser;
	}

	public void setCreatingUser(T creatingUser)
	{
		this.creatingUser = creatingUser;
	}

	public Date getLastModifiedDate()
	{
		return lastModifiedDate;
	}

	public void setLastModifiedDate(Date lastModifiedDate)
	{
		this.lastModifiedDate = lastModifiedDate;
	}

	public T getLastModifyingUser()
	{
		return lastModifyingUser;
	}

	public void setLastModifyingUser(T lastModifyingUser)
	{
		this.lastModifyingUser = lastModifyingUser;
	}
}