package com.portlandwebworks.commons.excel;

import com.portlandwebworks.commons.reflection.FieldUtils;
import org.apache.poi.hssf.usermodel.*;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class ApachePoiExcel implements IExcel
{
	public byte[] render(Workbook workbook, Map<String, List<?>> data)
	{
		HSSFWorkbook hssfWorkbook = new HSSFWorkbook();

		for (Worksheet worksheet : workbook.getWorksheets()) {
			
			HSSFSheet hssfSheet = hssfWorkbook.createSheet(worksheet.getName());
			
			int currRow = 0;

			for (Component component : worksheet.getComponents()) {

				for (int i = 0; i < component.getRowPadding(); i++) {
					hssfSheet.createRow(currRow++);
				}

				if (component instanceof Table) {
					Table table = (Table)component;

					HSSFRow headerRow = hssfSheet.createRow(currRow++);
					
					int currColumn = 0;
					while (currColumn < table.getColumnPadding()) {
						headerRow.createCell(currColumn++);
					}

					for (Column column : table.getColumns()) {
						HSSFCell headerCell = headerRow.createCell(currColumn++);
						HSSFRichTextString rtf = new HSSFRichTextString(column.getText());
						HSSFFont font = hssfWorkbook.createFont();
						font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
						font.setFontName("Verdana");
						font.setFontHeightInPoints((short) 10);
						font.setColor(HSSFFont.COLOR_NORMAL);
						rtf.applyFont(font);
						headerCell.setCellValue(rtf);
					}

					List<?> tableData = data.get(table.getDataSource());
					if (tableData == null) {
						throw new RuntimeException("Data source not found: " + table.getDataSource());
					}
					
					for (Object datum : tableData) {

						HSSFRow row = hssfSheet.createRow(currRow++);

						currColumn = 0;
						while (currColumn < table.getColumnPadding()) {
							headerRow.createCell(currColumn++);
						}
						
						for (Column column : table.getColumns()) {
							HSSFCell cell = row.createCell(currColumn++);
							Object fieldValue = FieldUtils.getFieldValue(datum, column.getDataIndex());
							HSSFRichTextString rtf = new HSSFRichTextString(fieldValue.toString());
							cell.setCellValue(rtf);
						}
					}
				}
			}
		}

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		try {
			hssfWorkbook.write(baos);
		}
		catch (IOException e) {
			throw new RuntimeException("error", e);
		}

		return baos.toByteArray();
	}
}
