/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.security.user;

import com.portlandwebworks.commons.dao.PagedSearchResult;
import com.portlandwebworks.commons.dao.PredicateSearchCriteria;
import com.portlandwebworks.commons.domain.User;
import com.portlandwebworks.commons.service.ServiceOperationResult;

import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public interface IUserService
{
	/**
	 *
	 *
	 *
	 * @param input
	 * @return
	 * @throws com.portlandwebworks.commons.security.user.DuplicateLoginException
	 */
	ServiceOperationResult<Long> addUser(UserInput input) throws DuplicateLoginException;

	/**
	 *
	 * @param userId
	 * @param input
	 */
	ServiceOperationResult<Void> updateUser(long userId, UserInput input);

	/**
	 * Returns a bunch of user input options
	 * 
	 * @return
	 */
	UserInputOptions getUserInputOptions();

	/**
	 *
	 *
	 * @param userId
	 * @return
	 */
	UserInputOptions getUserInputOptions(Long userId);

	/**
	 * Returns a single list of user input options of the type indicated, narrowed down by an optional qualifier.
	 *
	 *
	 *
	 * @param optionName
	 * @return
	 */
	List<?> getUserInputOptions(String optionName, String qualifier);

	/**
	 *
	 * @return
	 */
	UserSearchOptions getUserSearchOptions();

	/**
	 * Used to retrieve search options for querying on fields added to a user subclass.
	 *
	 *
	 * @param optionName
	 * @return
	 */
	List getUserSearchOptions(String optionName, String qualifier);

	/**
	 *
	 *
	 * @param criteria
	 * @return
	 */
	PagedSearchResult<? extends UserSearchResult> searchUsers(PredicateSearchCriteria criteria);

	/**
	 *
	 * @param login
	 * @return
	 */
	User getUserByLogin(String login);

	/**
	 *
	 * @param userId
	 * @return
	 */
	UserInput getUserInput(long userId);

	/**
	 * Initiates the password reset process.
	 *
	 * @param username
	 */
	ServiceOperationResult<Void> resetUserPassword(String username);

	/**
	 * Changes the user's password through a token system.
	 *
	 * @param input
	 */
	ServiceOperationResult<Void> changeUserPassword(ChangePasswordInput input);

	/**
	 * Gets the entity representation of a user.
	 *
	 * @param userId
	 * @return
	 */
	User getUser(long userId);

	/**
	 *
	 * @param username
	 * @param password
	 */
	void changeUserPassword(String username, String password);
}
