package com.portlandwebworks.commons.service;

import java.util.HashMap;
import java.util.Map;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class ServiceOperationResult<T>
{
	private T item;

	/**
	 *  Success property of transaction. true/false to be interpreted by client
	 */
	private Boolean success;

	/**
	 *  General message describing the success or failure of the service operation.  Detailed messages should be place
	 * in the errors field.
	 */
	private String message;

	/**
	 * Input validation errors, as well as any business logic errors associated with a service operation.  Errors should
	 * be keyed by the name of the input field that caused them.  If the error is not specific to any field, just make
	 * up a key for it.
	 */
	private Map<String,String> errors = new HashMap<String, String>();
	
	public static final ServiceOperationResult<Void> simpleSuccessfulResult =
			new ServiceOperationResult<Void>(true);


	/**
	 * Creates a result with none of its fields populated.
	 */
	public ServiceOperationResult()
	{
	}

	/**
	 * Creates a result that contains no data item and no message.
	 *
	 * @param success
	 */
	public ServiceOperationResult(Boolean success)
	{
		this.success = success;
	}

	/**
	 * Creates a result that contains no data item.
	 *
	 * @param success
	 * @param message
	 */
	public ServiceOperationResult(Boolean success, String message)
	{
		this.success = success;
		this.message = message;
	}

	/**
	 * Creates a result that contains an item and a success indicator.
	 * @param item
	 * @param success
	 */
	public ServiceOperationResult(T item, Boolean success)
	{
		this.item = item;
		this.success = success;
	}

	/**
	 *
	 * @param item
	 * @param success
	 * @param message
	 */
	public ServiceOperationResult(T item, Boolean success, String message)
	{
		this.item = item;
		this.success = success;
		this.message = message;
	}

	/**
	 *
	 * @param item
	 * @param success
	 * @param message
	 * @param errors
	 */
	public ServiceOperationResult(T item, Boolean success, String message, Map<String, String> errors)
	{
		this.item = item;
		this.success = success;
		this.message = message;
		this.errors = errors;
	}

	public T getItem()
	{
		return item;
	}

	public void setItem(T item)
	{
		this.item = item;
	}

	public Boolean getSuccess()
	{
		return success;
	}

	public void setSuccess(Boolean success)
	{
		this.success = success;
	}

	public String getMessage()
	{
		return message;
	}

	public void setMessage(String message)
	{
		this.message = message;
	}

	public Map<String, String> getErrors()
	{
		return errors;
	}

	public void setErrors(Map<String, String> errors)
	{
		this.errors = errors;
	}
}

