package com.portlandwebworks.commons.spring.security;

import com.portlandwebworks.commons.service.JsonObjectMapper;
import com.portlandwebworks.commons.service.ServiceOperationResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**z
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class JsonAuthenticationEntryPoint implements AuthenticationEntryPoint
{

    @Autowired
    private JsonObjectMapper jsonMapper;

    public void commence(HttpServletRequest request, HttpServletResponse response,
                         AuthenticationException authException) throws IOException, ServletException
    {

        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);

        ServiceOperationResult<Void> result = new ServiceOperationResult<Void>(
                null,
                false,
                "Authentication session has expired.  Please Reauthenticate");

        response.setContentType("application/json;charset=UTF-8");

        PrintWriter writer = response.getWriter();

        jsonMapper.writeValue(writer, result);

        writer.close();

    }

    public JsonObjectMapper getJsonMapper() {
        return jsonMapper;
    }

    public void setJsonMapper(JsonObjectMapper jsonMapper) {
        this.jsonMapper = jsonMapper;
    }
}