package com.portlandwebworks.commons.test;

import com.portlandwebworks.commons.dao.DataIntegrityException;
import com.portlandwebworks.commons.dao.DuplicateDataException;
import com.portlandwebworks.commons.dao.IEntityManager;
import com.portlandwebworks.commons.test.dbunit.DbUnitTestExecutionListener;
import com.portlandwebworks.commons.test.security.AuthenticationTestExecutionListener;
import org.junit.After;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.TestExecutionListeners;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.support.DependencyInjectionTestExecutionListener;
import org.springframework.test.context.transaction.AfterTransaction;
import org.springframework.test.context.transaction.BeforeTransaction;
import org.springframework.test.context.transaction.TransactionConfiguration;
import org.springframework.test.context.transaction.TransactionalTestExecutionListener;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * Base class for unit testing database-level code.  Relies on the presence of a spring context named text-context.xml
 * being available via class loader.  To have Spring provide a transaction for your unit tests, add a @Transactional
 * annotation to the test method signature.
 *
 * @author Ronnie Northup
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath*:test-context.xml" })
@TransactionConfiguration(defaultRollback = false)
@Transactional(propagation = Propagation.REQUIRED, rollbackFor = {
		DuplicateDataException.class, DataIntegrityException.class
})
@TestExecutionListeners({
		DependencyInjectionTestExecutionListener.class,
		DbUnitTestExecutionListener.class,
		TransactionalTestExecutionListener.class,
		AuthenticationTestExecutionListener.class
})
public class AbstractDatabaseTest
{
	@Autowired
	protected IEntityManager entityManager;

	@Autowired
	protected PlatformTransactionManager txManager;

	@Autowired
	protected ApplicationContext applicationContext;

	@BeforeTransaction
	public void beforeTransaction()
	{

	}

	@AfterTransaction
	public void afterTransaction()
	{
		System.out.println("");
	}

	@Before
	public void beforeTests()
	{

	}

	@After
	public void afterTests()
	{
		System.out.println("");
	}
}

