package com.portlandwebworks.commons.validation;

import com.portlandwebworks.commons.reflection.FieldUtils;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class MatchesValidator implements ConstraintValidator<Matches, Object>
{

	private String field;
	private String verifyField;


	public void initialize(Matches constraintAnnotation)
	{
		this.field = constraintAnnotation.field();
		this.verifyField = constraintAnnotation.verifyField();
	}

	public boolean isValid(Object value, ConstraintValidatorContext context)
	{
		Object fieldObj = FieldUtils.getFieldValue(value, field);
		Object verifyFieldObj = FieldUtils.getFieldValue(value, verifyField);

		boolean neitherSet = (fieldObj == null) && (verifyFieldObj == null);

		if (neitherSet) {
			return true;
		}

		boolean matches = (fieldObj != null) && fieldObj.equals(verifyFieldObj);

		if (!matches) {
			context.disableDefaultConstraintViolation();
			context.buildConstraintViolationWithTemplate("message")
					.addNode(verifyField)
					.addConstraintViolation();
		}

		return matches;
	}
}